var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { throwError } from 'rxjs';
import { catchError, map } from 'rxjs/operators';
import { HttpHeaders } from '@angular/common/http';
import { ODataExecReturnType } from './angularODataEnums';
import { ODataOperation } from './angularODataOperation';
var ODataQuery = /** @class */ (function (_super) {
    __extends(ODataQuery, _super);
    function ODataQuery(typeName, config, http) {
        var _this = _super.call(this, typeName, config, http) || this;
        _this._orderBy = [];
        _this._apply = [];
        _this._entitiesUri = config.getEntitiesUri(_this.typeName);
        return _this;
    }
    ODataQuery.prototype.Filter = function (filter) {
        if (filter) {
            this._filter = filter;
        }
        return this;
    };
    ODataQuery.prototype.Search = function (search) {
        if (search) {
            this._search = search;
        }
        return this;
    };
    ODataQuery.prototype.Top = function (top) {
        if (top > 0) {
            this._top = top;
        }
        return this;
    };
    ODataQuery.prototype.Skip = function (skip) {
        if (skip > 0) {
            this._skip = skip;
        }
        return this;
    };
    ODataQuery.prototype.OrderBy = function (orderBy) {
        if (orderBy) {
            this._orderBy = this.toStringArray(orderBy);
        }
        return this;
    };
    ODataQuery.prototype.MaxPerPage = function (maxPerPage) {
        if (maxPerPage > 0) {
            this._maxPerPage = maxPerPage;
        }
        return this;
    };
    ODataQuery.prototype.Apply = function (apply) {
        if (apply) {
            this._apply = this.toStringArray(apply);
        }
        return this;
    };
    ODataQuery.prototype.GetUrl = function (returnType) {
        var url = this._entitiesUri;
        if (returnType === ODataExecReturnType.Count) {
            url = url + "/" + this.config.keys.count;
        }
        var params = this.getQueryParams(returnType === ODataExecReturnType.PagedResult);
        if (params.keys().length > 0) {
            return url + "?" + params;
        }
        return url;
    };
    ODataQuery.prototype.Exec = function (returnType) {
        var requestOptions = this.getQueryRequestOptions(returnType === ODataExecReturnType.PagedResult);
        switch (returnType) {
            case ODataExecReturnType.Count:
                return this.execGetCount(requestOptions);
            case ODataExecReturnType.PagedResult:
                return this.execGetArrayDataWithCount(this._entitiesUri, requestOptions);
            default:
                return this.execGetArrayData(requestOptions);
        }
    };
    ODataQuery.prototype.ExecWithCount = function () {
        return this.Exec(ODataExecReturnType.PagedResult);
    };
    ODataQuery.prototype.NextPage = function (pagedResult) {
        var requestOptions = this.getQueryRequestOptions(false);
        return this.execGetArrayDataWithCount(pagedResult.nextLink, requestOptions);
    };
    ODataQuery.prototype.execGetCount = function (requestOptions) {
        var _this = this;
        var countUrl = this._entitiesUri + "/" + this.config.keys.count;
        return this.http.get(countUrl, requestOptions)
            .pipe(map(function (res) { return _this.extractDataAsNumber(res, _this.config); }), catchError(function (err, caught) {
            if (_this.config.handleError) {
                _this.config.handleError(err, caught);
            }
            return throwError(err);
        }));
    };
    ODataQuery.prototype.execGetArrayDataWithCount = function (url, requestOptions) {
        var _this = this;
        return this.http.get(url, requestOptions)
            .pipe(map(function (res) { return _this.extractArrayDataWithCount(res, _this.config); }), catchError(function (err, caught) {
            if (_this.config.handleError) {
                _this.config.handleError(err, caught);
            }
            return throwError(err);
        }));
    };
    ODataQuery.prototype.execGetArrayData = function (requestOptions) {
        var _this = this;
        return this.http.get(this._entitiesUri, requestOptions)
            .pipe(map(function (res) { return _this.extractArrayData(res, _this.config); }), catchError(function (err, caught) {
            if (_this.config.handleError) {
                _this.config.handleError(err, caught);
            }
            return throwError(err);
        }));
    };
    ODataQuery.prototype.getQueryRequestOptions = function (odata4) {
        var options = Object.assign({}, this.config.defaultRequestOptions);
        options.params = this.getQueryParams(odata4);
        if (this._maxPerPage > 0) {
            if (!options.headers) {
                options.headers = new HttpHeaders();
            }
            options.headers = options.headers.set('Prefer', this.config.keys.maxPerPage + "=" + this._maxPerPage);
        }
        return options;
    };
    ODataQuery.prototype.getQueryParams = function (odata4) {
        var params = _super.prototype.getParams.call(this);
        if (this._filter) {
            params = params.append(this.config.keys.filter, this._filter);
        }
        if (this._search) {
            params = params.append(this.config.keys.search, this._search);
        }
        if (this._top > 0) {
            params = params.append(this.config.keys.top, this._top.toString());
        }
        if (this._skip > 0) {
            params = params.append(this.config.keys.skip, this._skip.toString());
        }
        if (this._orderBy.length > 0) {
            params = params.append(this.config.keys.orderBy, this.toCommaString(this._orderBy));
        }
        if (this._apply.length > 0) {
            params = params.append(this.config.keys.apply, this.toCommaString(this._apply));
        }
        if (odata4) {
            params = params.append('$count', 'true'); // OData v4 only
        }
        return params;
    };
    ODataQuery.prototype.extractDataAsNumber = function (res, config) {
        return config.extractQueryResultDataAsNumber(res);
    };
    ODataQuery.prototype.extractArrayData = function (res, config) {
        return config.extractQueryResultData(res);
    };
    ODataQuery.prototype.extractArrayDataWithCount = function (res, config) {
        return config.extractQueryResultDataWithCount(res);
    };
    return ODataQuery;
}(ODataOperation));
export { ODataQuery };
//# sourceMappingURL=angularODataQuery.js.map